<?php

namespace Controllers;

//use Teknisa\GER\Utilities\Environment;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
// use Zeedhi\Framework\DataSource\Manager;
// use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
// use Zeedhi\Framework\DTO\Response\Message;
use \Teknisa\GER\Services\CrudService;

class CustoProdutoCoc extends \Teknisa\GER\Controllers\TeknisaCrud{

    const SUB_RECEITA = "S";

    protected $dataSourceManager;
    protected $combinacoes = array();
    protected $parameterBag;
    protected $parameterBagColumns = array("CDFILIAL", "CODIGO", "DATACUSTO", "TIPOCUSTO");
    protected $environment;

    public function __construct($crudService, $parameterBag, $dataSourceManager, $environment) {
        $this->dataSourceManager = $dataSourceManager;
        $this->parameterBag = $parameterBag;
        $this->environment=$environment;
        parent::__construct($crudService, $parameterBag);
    }


	public function findCusto(DTO\Request\DataSet $request, DTO\Response $response) {
        foreach($request->getDataSet()->getRows() as $row) {

            if($row["ID"] == self::SUB_RECEITA) {
                $dataSourceName = 'custo_receitam_coc';
                $filterCriteria = new FilterCriteria($dataSourceName);
                $filterCriteria->addCondition("NRVARIACAO", "=", $row['NRVARIACAO']);
                $this->parameterBagColumns = array("CDFILIAL", "CODIGO", "NRVARIACAO", "DATACUSTO", "TIPOCUSTO");
                
            }else {
                $dataSourceName = 'custo_produto_coc';
                $filterCriteria = new FilterCriteria($dataSourceName);
            }


            $filterCriteria->addCondition("CDFILIAL", "=", $row['CDFILIAL']);
            $filterCriteria->addCondition("CODIGO", "=", $row['CODIGO']);
            $filterCriteria->addCondition("DATACUSTO", "=", $row['DATACUSTO']);
            $filterCriteria->addCondition("TIPOCUSTO", "=", $row['TIPOCUSTO']);

            $newFilterCriteria = $this->populateParameterBag($filterCriteria, $this->parameterBagColumns);

            $combinacao = $this->dataSourceManager->findBy($newFilterCriteria);

            $this->combinacoes[] = array(
                'ID' => $row['ID'],
                'CDFILIAL' => $row['CDFILIAL'],
                'CODIGO' => $row['CODIGO'],
                'CUSTO' => $combinacao->getRows()[0]['CUSTO']
            );
 
        }
        $this->verificaAlteracoes($request, $response);
        $response->addDataSet(new DataSet ('composicao', $this->combinacoes));
	}

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    public function populateParameterBag(FilterCriteria $filterCriteria, $parameterBagColumns) {

        $conditions = array();

        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );

        return $newFilterCriteria;
    }
    public function verificaAlteracoes(DTO\Request $request, DTO\Response $response){
        foreach($request->getDataSet()->getRows() as $row) { 
            
            $pkCDRECEITA = $row['CDRECEITA'];
            $pkNRVARIACAOREC = $row['NRVARIACAOREC'];
            
            $rowReceita = array(
                'CDRECEITA' => $pkCDRECEITA,
                '__is_new' => false
            );
            $this->salvaAlteracoes($request,$response, $rowReceita, 'receitam');
            $rowVariacao = array(
                'CDRECEITA' => $pkCDRECEITA,
                'NRVARIACAOREC' => $pkNRVARIACAOREC,
                '__is_new' => false
            );
            $this->salvaAlteracoes($request,$response, $rowVariacao, 'variacaorec');
        }
    }
    
    private function salvaAlteracoes(DTO\Request $request, DTO\Response $response, $row, $dataSourceName){
        
        $dataAtual = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $operador = $this->environment->getCdOperador();
        $row['DTULTATU'] = $dataAtual;
        $row['DSOPERULTATU'] = $operador;
    
        $method = $request->getMethod();
        $routePath = $request->getRoutePath();
        $userId = $request->getUserId();

        $newRequest = new DTO\Request\Row($row, $method, $routePath, $userId);
        $rows = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
		$this->crudService->save($dataSourceName, $rows);
        $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
        
    }


}