<?php

namespace Controllers;

// use Teknisa\GER\Utilities\Environment;
// use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
// use Zeedhi\Framework\DTO\Response\Message;

class FrequenciaCicloVooController extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $dataSourceName = 'frciclovoo_anual';
    protected $parameterBagColumns = array("DTINICIAL", "DTFINAL", "NRORG", "CDFILIAL", "CDCIA");
	protected $connection;


    public function __construct($crudService, $parameterBag,  $environment, $dataSourceManager, $connection) {
        $this->crudService = $crudService;
        $this->parameterBag=$parameterBag;
        $this->environment=$environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->connection = $connection;
    }
    
    public function importaCicloVoo(DTO\Request $request, DTO\Response $response){
    	$dataSource = 'frciclovoo';
    	try{
    		$this->connection->beginTransaction();
	    	$row = $request->getRow();
			$frequenciaNoPeriodo = $this->buscaFrequenciasNoPeriodo($row);
			
			$rowsASeremSalvas = $this->preparaRows($frequenciaNoPeriodo, $row);
			$this->apagaFrequenciaNoPeriodo($row);
			
			$newDataSet = new DataSet($dataSource, $rowsASeremSalvas);
			$newRequest = new DTO\Request\DataSet($newDataSet, "", "", "");
			$rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSource);
			$dataSet = new DataSet($dataSource, $rowsToSave);
			$this->dataSourceManager->persist($dataSet);
			$this->connection->commit();
    	} catch(\Exception $e){
    		$this->connection->rollback();
    		$response->setError(new Error($e->getMessage(), 500));
    	}
		
    }
    
    public function exportaCicloVoo(DTO\Request $request, DTO\Response $response){
    	$dataSource = 'frciclovoo';
    	try{
    		$this->connection->beginTransaction();
			$row = $request->getRow();
    		for($i = 0; $i < count($row['CHAVE']); $i++){
    			$chave = explode("_",$row['CHAVE'][$i]);
    			$row['NRVOODES'] = $chave[0]; 
    			$row['CDTRECHODES'] = $chave[1]; 
				
				$frequenciaNoPeriodo = $this->buscaFrequenciasNoPeriodo($row);
				$rowsASeremSalvas = $this->preparaRows($frequenciaNoPeriodo, $row);
				
				$this->apagaFrequenciaNoPeriodo($row);
				$newDataSet = new DataSet($dataSource, $rowsASeremSalvas);
				$newRequest = new DTO\Request\DataSet($newDataSet, "", "", "");
				$rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSource);
				$dataSet = new DataSet($dataSource, $rowsToSave);
				$this->dataSourceManager->persist($dataSet);
    		}
    		$this->connection->commit();
		} catch(\Exception $e){
			$this->connection->rollback();
    		$response->setError(new Error($e->getMessage(), 500));
    	}
    }
    
    private function buscaFrequenciasNoPeriodo($row){
    	
    	$query = 'SELECT CDFILIAL,
					     CDCIA,
					     CDCICLO,
					     NRVOO,
					     CDTRECHO,
					     DTCICLOVOO,
					     NRORG
					FROM FRCICLOVOO
				   WHERE CDFILIAL = :CDFILIAL
					 AND CDCIA = :CDCIA
					 AND NRVOO = :NRVOO
					 AND CDTRECHO = :CDTRECHO
					 AND DTCICLOVOO BETWEEN :DTINICIAL AND :DTFINAL
					 AND NRORG = :NRORG';
		$stmt = $this->connection->prepare($query);
		$params = array(
			'CDFILIAL'  => $row['CDFILIAL'],
			'CDCIA'     => $row['CDCIA'],
			'NRVOO'     => $row['NRVOO'],
			'CDTRECHO'  => $row['CDTRECHO'],
			'DTINICIAL' => $row['DTINIVGMA'],
			'DTFINAL'   => $row['DTFIMVGMA'],
			'NRORG' => $this->environment->getNrOrgTrab()
		);
		
		$stmt->execute($params);
		
		$result = $stmt->fetchAll(\PDO::FETCH_ASSOC);
		
		return $result;
    	
    }
    
    private function apagaFrequenciaNoPeriodo($row){
    	
    	$query = 'DELETE FROM FRCICLOVOO
                   WHERE CDFILIAL = :CDFILIAL
                     AND CDCIA = :CDCIA
                     AND CDTRECHO = :CDTRECHO
                     AND NRVOO = :NRVOO
                     AND DTCICLOVOO BETWEEN :DTINICIAL AND :DTFINAL
                     AND NRORG = :NRORG';
                     
		$stmt = $this->connection->prepare($query);
		
		$params = array(
			'CDFILIAL'  => $row['CDFILIAL'],
			'CDCIA'     => $row['CDCIA'],
			'NRVOO'     => $row['NRVOODES'],
			'CDTRECHO'  => $row['CDTRECHODES'],
			'DTINICIAL' => $row['DTINIVGMA'],
			'DTFINAL'   => $row['DTFIMVGMA'],
			'NRORG' => $this->environment->getNrOrgTrab()
		);
		
		$stmt->execute($params);
		
    }
    
	private function preparaRows($rowsImportadas, $row){
    	    	
		foreach ($rowsImportadas as &$item) {
			$item['CDTRECHO'] = $row['CDTRECHODES'];
			$item['NRVOO'] = $row['NRVOODES'];
			$item['CDCIA'] = $row['CDCIA'];
			$item['changed'] = false;
			$item['__is_new'] = true;
		}
		
		return $rowsImportadas;
	
    }
    
}
    