<?php
namespace Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\Report\Strategy\BirtStrategy;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class RelatoriodeReceitas extends \Zeedhi\Framework\Controller\Report {

    protected $environment;
    protected $birtStrategy;
    protected $connection;
    protected $crudService;
    protected $util;

    public function __construct(ReportService $reportService, $environment, $connection, $birtStrategy, $crudService, $util) {
        $this->birtStrategy = $birtStrategy;
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->connection = $connection;
        $this->util = $util;
        parent::__construct($reportService);
    }


    protected $reportMapping = array (
        'COC00991' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"            =>"P_NRORG",
              "NRLISTA"         =>"NRLISTA",
              "P_ZEBRADO"        =>"P_ZEBRADO",
              "__format"         =>"__format",
              "CDFILIAL" => "CDFILIAL",
              "CDFILIALLOGO"     =>"CDFILIALLOGO"
            )
        ),
        'COC0B991' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"            =>"NRORG",
              "NRLISTA"          =>"NRLISTA",
              "CDCATEGORIA"      =>"CDCATEGORIA",
              "CDSETOR"          =>"CDSETOR",
              "P_ZEBRADO"        =>"ZEBRADO",
              "CDFILIAL" => "CDFILIAL",
              "CDFILIALLOGO"     =>"CDFILIALLOGO"
            )
        ),
        'COC0C991' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"            =>"NRORG",
              "NRLISTA"          =>"NRLISTA",
              "CDSETOR"          =>"CDSETOR",
              "P_ZEBRADO"        =>"ZEBRADO",
              "CDFILIAL" => "CDFILIAL",
              "CDFILIALLOGO"     =>"CDFILIALLOGO"
            )
        ),
        'COC0D991' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"            =>"NRORG",
              "NRLISTA"          =>"NRLISTA",
              "CDSETOR"          =>"CDSETOR",
              "P_ZEBRADO"        =>"ZEBRADO",
              "CDFILIAL" => "CDFILIAL",
              "CDFILIALLOGO"     =>"CDFILIALLOGO"
            )
        ),
        'COC0E991' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"            =>"NRORG",
              "CDFILIAL"  =>"CDFILIAL",
              "CDCIA"            =>"CDCIA",
              "CDPRODUTO"        =>"CDPRODUTO",
              "P_ZEBRADO"        =>"ZEBRADO",
              "IDPRODATIVPR"     =>"IDPRODATIVPR",
              "IDSITUAPROPCIA"   =>"IDSITUAPROPCIA",
              "NRGERACAO"        =>"NRGERACAO"
            )
        ),
        'COC0K991' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"            =>"NRORG",
              "CDFILIAL"  => "CDFILIAL",
              "CDCIA"            =>"CDCIA",
              "CDPRODUTO"        =>"CDPRODUTO",
              "P_ZEBRADO"        =>"ZEBRADO",
              "IDPRODATIVPR"     =>"IDPRODATIVPR",
              "IDSITUAPROPCIA"   =>"IDSITUAPROPCIA",
              "NRGERACAO"        =>"NRGERACAO"
            )
        ),
        'COC0F991' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"            =>"NRORG",
              "CDRECEITA"        =>"CDRECEITA",
              "P_ZEBRADO"        =>"ZEBRADO",
              "CDFILIAL" => "CDFILIAL",
              "CDFILIALLOGO"     =>"CDFILIALLOGO"
            )
        ),
        'COC0G991' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"            =>"NRORG",
              "CDLEANCODE"       =>"CDLEANCODE",
              "P_ZEBRADO"        =>"ZEBRADO",
              "NRLISTA"          =>"NRLISTA",
              "CDFILIAL" => "CDFILIAL",
              "CDFILIALLOGO"     =>"CDFILIALLOGO"
            )
        ),
        'COC0L991' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"            =>"NRORG",
              "CDLEANCODE"       =>"CDLEANCODE",
              "P_ZEBRADO"        =>"ZEBRADO",
              "NRLISTA"          =>"NRLISTA",
              "CDFILIAL" => "CDFILIAL",
              "CDFILIALLOGO"     =>"CDFILIALLOGO"
            )
        ),
        'COC0H991' => array ( //Tempo de Mão de Obra por Receita
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"            =>"NRORG",
              "CDRECEITA"        =>"CDRECEITA",
              "CDFILIAL" => "CDFILIAL",
              "CDFILIALLOGO"     =>"CDFILIALLOGO"
              
            )
        ),
        'COC0I991' => array ( //Alteração de Receitas
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"           =>"NRORG",
              "IDTIPOPERWFLOW"  =>"P_IDTIPOPERWFLOW",
              "CDFILIAL" => "CDFILIAL",
              "CDFILIALLOGO"    => "CDFILIALLOGO"
            )
        ),
        'COC00992' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"            =>"NRORG",
              "CDRECEITA"        =>"CDRECEITA",
              "CDFILIAL"  =>"CDFILIAL"
            )
        ),
        'COC00993' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"            =>"NRORG",
              "CDRECEITA"        =>"CDRECEITA",
              "CDFILIAL" => "CDFILIAL",
              "CDFILIALLOGO"     =>"CDFILIALLOGO"
            )
        ),
        'COC02600' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"            =>"NRORG",
              "CDFILIAL" => "CDFILIAL",
              "CDSUBRECEITA"         =>"CDSUBRECEITA",
              "CDFILIALLOGO"         =>"CDFILIALLOGO"
            )
        ),
        'COC02700' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"            =>"NRORG",
              "CDFILIAL" => "CDFILIAL",
              "UNPRODUTO"        =>"CDPRODUTO",
              "CDFILIALLOGO"     =>"CDFILIALLOGO"
            )
        ),
        'COC02800' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"            =>"NRORG",
              "CDFILIAL" => "CDFILIAL",
              "CDFILIALLOGO"     =>"CDFILIALLOGO"
            )
        ),
        'COC02300' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"           => "NRORG",
              "CDFILIAL" => "CDFILIAL",
              "CDRECEITA"       => "CDRECEITA",
              "IDSITUAPROPCIA"  => "IDSITUAPROPCIA",
              "CDCIACDMANCIA"   => "CDCIACDMANCIA",
              "CDCIA"           => "CDCIA",
              "NRVARIACAOREC"   => "NRVARIACAOREC"
            )
        ),
        'COC0J991' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"            =>"NRORG",
              "CDFILIAL" => "CDFILIAL",
              "NRLISTA"          =>"NRLISTA",
              "CDSETOR"          =>"CDSETOR",
              "P_ZEBRADO"        =>"ZEBRADO",
              "CDFILIALLOGO"     =>"CDFILIALLOGO"
            )
        )
    );

  public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response){
    $row = $request->getRow();
    $this->birtStrategy->setReportFormat($row['__format']);
    $row['CDFILIALLOGO'] = $this->environment->getCdFilial();
    if($row['REPORTNAME'] !== 'COC00992' && $row['REPORTNAME'] !== 'COC0E991' && $row['REPORTNAME'] !== 'COC0H991') {
      if($row['CDFILIAL'] === 'T') {
        $params = array(
          array(
                  "columnName" => 'CDOPERADOR',
                  "operator"   => '=',
                  "value"      => $this->environment->getCdoperador()
          )
        );
        $units = $this->crudService->getFilteredDataSet('filial_filioper', $params)->getRows();
        $codUnit = [];
        foreach($units as $unit) {
          array_push($codUnit, $unit['CDFILIAL']);
        }
        $row['CDFILIAL'] = $codUnit;
      } else if(count($row['CDFILIAL']) === 1){
        $row['CDFILIALLOGO'] = $row['CDFILIAL'][0];
      }
      $row['NRLISTA'] = $this->util->insertLista($row['CDFILIAL']);
    }

    if ($row['REPORTNAME'] === 'COC0E991' || $row['REPORTNAME'] === 'COC0K991') {
      $geraAnaliseReceitam = $this->geraAnaliseReceitam($row, $response);
      $row['NRGERACAO'] = $geraAnaliseReceitam['P_NRGERACAO'];
    } else if ($row['REPORTNAME'] === 'COC02300') {
        $conditions = array(
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $row['CDFILIAL']
            ),
            array(
                'columnName' => 'CDRECEITA',
                'operator' => '=',
                'value' => $row['CDRECEITA']
            )
        );
        $nrvariacaoReceita = $this->crudService->getFilteredDataSet('filireceita', $conditions)->getRows();
        if(!empty($nrvariacaoReceita )){
            $row['NRVARIACAOREC'] = $nrvariacaoReceita[0]['NRVARIACAOREC'];
        } else {
            $row['NRVARIACAOREC'] = 'T';
        }
    }
  }

  private function geraAnaliseReceitam($row, $response) {
    try {
      $procedure = new StoredProcedure($this->connection, 'GERA_ANALISERECEITAM');
      $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row['CDFILIAL']));
      $procedure->addParam(new Param('P_NRGERACAO', Param::PARAM_OUTPUT, null, null, 10));
      $procedure->addParam(new Param('P_CDCIA', Param::PARAM_INPUT, null));
      $procedure->addParam(new Param('P_CDMANCIA', Param::PARAM_INPUT, null));
      $procedure->addParam(new Param('P_DTCUSTO', Param::PARAM_INPUT, null));
      $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, intval($this->environment->getNrOrgTrab())));
      $controle = $procedure->execute();
      return $controle;
    } catch (\Exception $e) {
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
}